@@ -1,5 +1,5 @@
 /* riemann/event.c -- Riemann C client library
- * Copyright (C) 2013, 2014  Gergely Nagy <algernon@madhouse-project.org>
+ * Copyright (C) 2013, 2014, 2015  Gergely Nagy <algernon@madhouse-project.org>
  *
  * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -15,13 +15,13 @@
  * License along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <riemann/attribute.h>
 #include <riemann/event.h>
 
 #include <errno.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <unistd.h>
 
 riemann_event_t *
 riemann_event_new (void)
@@ -50,7 +50,10 @@ _riemann_event_set_string (char **str, char *value)
 {
   if (*str)
     free (*str);
-  *str = strdup (value);
+  if (value)
+    *str = strdup (value);
+  else
+    *str = NULL;
 }
 
 int
@@ -107,7 +110,7 @@ riemann_event_set_va (riemann_event_t *event,
 
             while ((tag = va_arg (ap, char *)) != NULL)
               {
-                event->tags =
+                event->tags = (char **)
                   realloc (event->tags, sizeof (char *) * (event->n_tags + 1));
                 event->tags[event->n_tags] = strdup (tag);
                 event->n_tags++;
@@ -135,7 +138,7 @@ riemann_event_set_va (riemann_event_t *event,
 
             while ((attrib = va_arg (ap, riemann_attribute_t *)) != NULL)
               {
-                event->attributes =
+                event->attributes = (riemann_attribute_t **)
                   realloc (event->attributes,
                            sizeof (riemann_attribute_t *) * (event->n_attributes + 1));
                 event->attributes[event->n_attributes] = attrib;
@@ -145,6 +148,33 @@ riemann_event_set_va (riemann_event_t *event,
             break;
           }
 
+        case RIEMANN_EVENT_FIELD_STRING_ATTRIBUTES:
+          {
+            const char *key, *value;
+            size_t n;
+
+            for (n = 0; n < event->n_attributes; n++)
+              riemann_attribute_free (event->attributes[n]);
+            if (event->attributes)
+              free (event->attributes);
+            event->attributes = NULL;
+            event->n_attributes = 0;
+
+            while ((key = va_arg (ap, const char *)) != NULL)
+              {
+                value = va_arg (ap, const char *);
+
+                event->attributes = (riemann_attribute_t **)
+                  realloc (event->attributes,
+                           sizeof (riemann_attribute_t *) * (event->n_attributes + 1));
+                event->attributes[event->n_attributes] =
+                  riemann_attribute_create (key, value);
+                event->n_attributes++;
+              }
+
+            break;
+          }
+
         case RIEMANN_EVENT_FIELD_METRIC_S64:
           event->metric_sint64 = va_arg (ap, int64_t);
           event->has_metric_sint64 = 1;
@@ -166,7 +196,7 @@ riemann_event_set_va (riemann_event_t *event,
         }
 
       if (field != RIEMANN_EVENT_FIELD_NONE)
-        field = va_arg (ap, riemann_event_field_t);
+        field = (riemann_event_field_t) va_arg (ap, int);
     }
   while (field != RIEMANN_EVENT_FIELD_NONE);
   va_end (ap);
@@ -182,7 +212,7 @@ riemann_event_set (riemann_event_t *event, ...)
   riemann_event_field_t first_field;
 
   va_start (ap, event);
-  first_field = va_arg (ap, riemann_event_field_t);
+  first_field = (riemann_event_field_t) va_arg (ap, int);
   r = riemann_event_set_va (event, first_field, ap);
   va_end (ap);
   return r;
@@ -194,7 +224,7 @@ riemann_event_tag_add (riemann_event_t *event, const char *tag)
   if (!event || !tag)
     return -EINVAL;
 
-  event->tags =
+  event->tags = (char **)
     realloc (event->tags, sizeof (char *) * (event->n_tags + 1));
   event->tags[event->n_tags] = strdup (tag);
   event->n_tags++;
@@ -209,7 +239,7 @@ riemann_event_attribute_add (riemann_event_t *event,
   if (!event || !attrib)
     return -EINVAL;
 
-  event->attributes =
+  event->attributes = (riemann_attribute_t **)
     realloc (event->attributes,
              sizeof (riemann_attribute_t *) * (event->n_attributes + 1));
   event->attributes[event->n_attributes] = attrib;
@@ -218,16 +248,28 @@ riemann_event_attribute_add (riemann_event_t *event,
   return 0;
 }
 
+int
+riemann_event_string_attribute_add (riemann_event_t *event,
+                                    const char *key,
+                                    const char *value)
+{
+  if (!key || !value)
+    return -EINVAL;
+
+  return riemann_event_attribute_add (event,
+                                      riemann_attribute_create (key, value));
+}
+
 riemann_event_t *
-riemann_event_create (riemann_event_field_t field, ...)
+riemann_event_create_va (riemann_event_field_t field, va_list aq)
 {
-  riemann_event_t *event;
   va_list ap;
+  riemann_event_t *event;
   int e;
 
   event = riemann_event_new ();
 
-  va_start (ap, field);
+  va_copy (ap, aq);
   if ((e = riemann_event_set_va (event, field, ap)) != 0)
     {
       va_end (ap);
@@ -239,3 +281,63 @@ riemann_event_create (riemann_event_field_t field, ...)
 
   return event;
 }
+
+riemann_event_t *
+riemann_event_create (riemann_event_field_t field, ...)
+{
+  riemann_event_t *event;
+  va_list ap;
+
+  va_start (ap, field);
+  event = riemann_event_create_va (field, ap);
+  va_end (ap);
+
+  return event;
+}
+
+riemann_event_t *
+riemann_event_clone (const riemann_event_t *event)
+{
+  riemann_event_t *clone;
+  size_t n;
+
+  if (!event)
+    {
+      errno = EINVAL;
+      return NULL;
+    }
+
+  clone = riemann_event_new ();
+
+  /* Copy non-pointer properties */
+  clone->time = event->time;
+  clone->ttl = event->ttl;
+  clone->metric_sint64 = event->metric_sint64;
+  clone->metric_d = event->metric_d;
+  clone->metric_f = event->metric_f;
+
+  /* Copy strings */
+  if (event->state)
+    clone->state = strdup (event->state);
+  if (event->host)
+    clone->host = strdup (event->host);
+  if (event->service)
+    clone->service = strdup (event->service);
+  if (event->description)
+    clone->description = strdup (event->description);
+
+  /* Copy deeper structures */
+  clone->n_tags = event->n_tags;
+  clone->tags = (char **) malloc (sizeof (char *) * clone->n_tags);
+  for (n = 0; n < clone->n_tags; n++)
+    clone->tags[n] = strdup (event->tags[n]);
+
+  clone->n_attributes = event->n_attributes;
+  clone->attributes = (riemann_attribute_t **)
+    malloc (sizeof (riemann_attribute_t *) *
+            clone->n_attributes);
+  for (n = 0; n < clone->n_attributes; n++)
+    clone->attributes[n] = riemann_attribute_clone (event->attributes[n]);
+
+  return clone;
+}
